/*
 * Copyright (C) 2005-2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file wav.c a filter that inserts a wave header */

#include "gcc-compat.h"
#include "para.h"

#include "list.h"
#include "filter.h"
#include "string.h"

#define WAV_OUTBUF_SIZE 81920
#define WAV_HEADER_LEN 44
#define BITS 16

/* derived from oggdec.c of the vorbis-tools-1.0.1 package */
#define WRITE_U32(buf, x) (buf)[0] = (unsigned char)((x) & 0xff);\
	(buf)[1] = (unsigned char)(((x) >> 8) & 0xff);\
	(buf)[2] = (unsigned char)(((x) >> 16) & 0xff);\
	(buf)[3] = (unsigned char)(((x) >> 24) & 0xff);
#define WRITE_U16(buf, x) (buf)[0] = (unsigned char)((x) & 0xff);

static void make_wav_header(int channels, int samplerate, struct filter_node *fn)
{

	char *headbuf = fn->buf;
	unsigned int size = 0x7fffffff;
	int bytespersec = channels * samplerate * BITS / 8;
	int align = channels * BITS / 8;

	PARA_DEBUG_LOG("writing wave header: %d channels, %d KHz\n", channels, samplerate);
	memset(headbuf, 0, WAV_HEADER_LEN);
	memcpy(headbuf, "RIFF", 4);
	WRITE_U32(headbuf + 4, size - 8);
	memcpy(headbuf + 8, "WAVE", 4);
	memcpy(headbuf + 12, "fmt ", 4);
	WRITE_U32(headbuf + 16, 16);
	WRITE_U16(headbuf + 20, 1);	/* format */
	WRITE_U16(headbuf + 22, channels);
	WRITE_U32(headbuf + 24, samplerate);
	WRITE_U32(headbuf + 28, bytespersec);
	WRITE_U16(headbuf + 32, align);
	WRITE_U16(headbuf + 34, BITS);
	memcpy(headbuf + 36, "data", 4);
	WRITE_U32(headbuf + 40, size - 44);
}

static ssize_t wav_convert(char *inbuf, size_t len, struct filter_node *fn)
{
	size_t copy;
	int *bof = fn->private_data;

	if (*bof) {
		make_wav_header(fn->fci->channels, fn->fci->samplerate, fn);
		fn->loaded = WAV_HEADER_LEN;
		*bof = 0;
//		return 0;
	}
	copy = MIN(len, fn->bufsize - fn->loaded);
	memmove(fn->buf + fn->loaded, inbuf, copy);
	fn->loaded += copy;
//	PARA_DEBUG_LOG("len = %d, copy = %d\n", len, copy);
	return copy;
}

static void wav_close(struct filter_node *fn)
{
	free(fn->buf);
	fn->buf = NULL;
	free(fn->private_data);
	fn->private_data = NULL;
}

static void wav_open(struct filter_node *fn)
{
	int *bof;

	fn->bufsize = WAV_OUTBUF_SIZE;
	fn->buf = para_malloc(fn->bufsize);
	fn->private_data = para_malloc(sizeof(int));
	bof = fn->private_data;
	*bof = 1;
	PARA_DEBUG_LOG("wav filter node: %p, output buffer: %p, loaded: %d\n",
		fn, fn->buf, fn->loaded);
}

void wav_init(struct filter *f)
{
	f->convert = wav_convert;
	f->close = wav_close;
	f->open = wav_open;
}
